#ifndef __ADFunction_H__
#define __ADFunction_H__

#include <sstream>
using namespace std;

#include "Function.h"
#include "Variable.h"
#include "GPTree.h"
#include "auto_list.h"


template<class T> class ADFunction : public Function<T> {

public:
	ADFunction( const char* name, int arity );
	explicit ADFunction( ADFunction& original );
	virtual ~ADFunction();
	
public:
	void set( GPTree<T>& tree );
	void execute( const auto_list<T>& inputs, T& output );
	vector<T*> inputs();
	
	      auto_list< Function<T> >& terminals()       { return original_->terminals_; };
	const auto_list< Function<T> >& terminals() const { return original_->terminals_; };
	      auto_list< Function<T> >& functions()       { return original_->functions_; };
	const auto_list< Function<T> >& functions() const { return original_->functions_; };
	
protected:
	GPTree<T>* tree_;
	auto_list< Function<T> > terminals_;
	auto_list< Function<T> > functions_;
	auto_list< T > inputs_;
	
	ADFunction<T>* original_;
};

template<class T> ADFunction<T>::ADFunction( const char* name, int arity )
 : Function<T>::Function( name, arity, true ),
   tree_( NULL ),
   original_( NULL )
{
	for ( int i = 0; i < arity; i++ ) {
		ostringstream strout;
		strout << "input" << i;
		inputs_.add( new T() );
		terminals_.add( new Variable<T>( strout.str().c_str(), inputs_[ i ] ) );
	}
	original_ = this;
}

template<class T> ADFunction<T>::ADFunction( ADFunction& original )
 : Function<T>::Function( original.name(), original.arity(), true ),
   tree_( NULL ),
   original_( &original )
{

}

template<class T> ADFunction<T>::~ADFunction() {
	tree_ = NULL;
}

template<class T> void ADFunction<T>::set( GPTree<T>& tree ) { 
	tree_ = &tree;
}

template<class T> void ADFunction<T>::execute( const auto_list<T>& inputs, T& output ) {
	if ( this != original_ ) original_->execute( inputs, output );
	else {
		assert( tree_ != NULL );
		for ( int i = 0; i < arity(); i++ )
			inputs_[ i ] = inputs[ i ];
		tree_->evaluate( output );
	}
};

template<class T> vector<T*> ADFunction<T>::inputs() {
	if ( this != original_ ) 
		return original_->inputs();
	vector<T*> v;
	for ( int i = 0; i < arity(); i++ )
		v.push_back( &inputs_[ i ] );
	return v;
}

#endif // __ADFunction_H__
