#ifndef __Candidate_H__
#define __Candidate_H__

#include <iostream>
#include <vector>
#include <limits>
using namespace std;


#include "GPProgram.h"
#include "Evaluation.h"
#include "ADFunction.h"
#include "rand-util.h"
#include "auto_list.h"

template<class T> class Candidate {

public:
	Candidate( Evaluation<T>& evaluation, auto_list< ADFunction<T> >& definedFunctions, bool initialise = true );
	virtual ~Candidate();
	
public:
	double fitness();
	void force_evaluation() { fitnessValid_ = false; };
	
	int program_size() const;
	
	      GPProgram<T>& program()       { return program_; };
	const GPProgram<T>& program() const { return program_; };
	
	void mutate();
	void recombine( Candidate<T>& partner );

	void print( ostream& output ) const; 

public:
	void operator=( const Candidate<T>& rhs );
	bool operator==( const Candidate<T>& rhs ) const;
	
public:
	static int    MAX_DEPTH;
	
	static double MUTATE_SUBTREE;
	static double MUTATE_TERMINAL;
	static double MUTATE_ROOT;
	static double MUTATE_FUNCTION;

protected:
	GPProgram<T> program_;
	double fitness_;
	bool fitnessValid_;
	Evaluation<T>* evaluation_;
};

template<class T> int Candidate<T>::MAX_DEPTH = 5;

template<class T> double Candidate<T>::MUTATE_SUBTREE  = 1.0;
template<class T> double Candidate<T>::MUTATE_TERMINAL = 0.1;
template<class T> double Candidate<T>::MUTATE_ROOT     = 0.1;
template<class T> double Candidate<T>::MUTATE_FUNCTION = 0.1;

template<class T> Candidate<T>::Candidate( Evaluation<T>& evaluation, auto_list< ADFunction<T> >& definedFunctions, bool initialise )
 : program_( MAX_DEPTH, definedFunctions, initialise ),
   fitness_( DBL_MAX ),
   fitnessValid_( false ),
   evaluation_( &evaluation )
{

}

template<class T> Candidate<T>::~Candidate() {

}


template<class T> int Candidate<T>::program_size() const {
	return program_.size();
}

template<class T> double Candidate<T>::fitness() {
	if ( !fitnessValid_ ) {
		assert( evaluation_ != NULL );
		fitness_ = evaluation_->evaluate( *this );
		fitnessValid_ = true;
	}
	return fitness_;
}



template<class T> void Candidate<T>::mutate() {
	int treeIndex = randrange( program_.num_trees() );
	GPTree<T>& tree = program_.tree( treeIndex );
	// ratio of doing one instead of another
	double sum = MUTATE_SUBTREE + MUTATE_TERMINAL + MUTATE_ROOT + MUTATE_FUNCTION;
	double r = randdouble()*sum;
	if ( r < MUTATE_SUBTREE )
		tree.mutate_subtree();
	else if ( r < (MUTATE_SUBTREE+MUTATE_TERMINAL) )
		tree.mutate_terminal();
	else if ( r < (MUTATE_SUBTREE+MUTATE_TERMINAL+MUTATE_FUNCTION) )
		tree.mutate_root();
	else
		tree.mutate_function();
	fitnessValid_ = false;
}

template<class T> void Candidate<T>::recombine( Candidate<T>& partner ) {
	int treeIndex = randrange( program_.num_trees() );
	GPTree<T>& tree1 = program_.tree( treeIndex );
	GPTree<T>& tree2 = partner.program_.tree( treeIndex );
	tree1.recombine( tree2 );
	fitnessValid_ = false;
};

template<class T> void Candidate<T>::operator=( const Candidate<T>& rhs ) {
	if ( this == &rhs )
		return;
	program_      = rhs.program_;
	fitness_      = rhs.fitness_;
	fitnessValid_ = rhs.fitnessValid_;
}

template<class T> bool Candidate<T>::operator==( const Candidate<T>& rhs ) const {
	if ( this == &rhs )
		return true;
	
	return program_ == rhs.program_;
}

template<class T> void Candidate<T>::print( ostream& output ) const {
	output << program_ << endl;
}

template<class T> ostream& operator<<(ostream& output, const Candidate<T>& candidate ) {
	candidate.print( output );
    return output;
}

#endif // __Candidate_H__
