#ifndef __FunctionNode_H__
#define __FunctionNode_H__

#include "GPNode.h"
#include "ConstantNode.h"

#include <iostream>
#include <strstream>
#include <memory>
using namespace std;

template<class T> class FunctionNode;

template<class T> class FunctionNode : public GPNode<T> {

public:
	FunctionNode( const FunctionNode<T>& root, const char* name, int arity );
	virtual ~FunctionNode();
	
public:
	int arity() const { return arity_; }
	const char* name() const { return name_; }

public:
    void print( ostream& output ) const;
	void read( istream& intput );

protected:
	void executeInputs();

protected:
	virtual FunctionNode<T>* makeFunction( const char* name ) const;

protected:
	const char* name_;
	const int arity_;
	auto_ptr< GPNode<T> >* inputs_;

protected:
    const FunctionNode<T>* root_;
};

///////////////////////
// implementation
///////////////////////

template<class T> FunctionNode<T>::FunctionNode( const FunctionNode<T>& root, const char* name, int arity )
 : GPNode<T>::GPNode(),
   name_( name ),
   arity_( arity ),
   root_( &root )
{
	inputs_ = new auto_ptr< GPNode<T> >[ arity_ ];
}

template<class T> FunctionNode<T>::~FunctionNode() {
	if ( inputs_ != NULL ) {
		delete[] inputs_;
		inputs_ = NULL;
		root_ = NULL;
	}
}

template<class T> FunctionNode<T>* FunctionNode<T>::makeFunction( const char* name ) const {
    if ( root_ != NULL ) {
        return root_->makeFunction( name );
    }
    return NULL;
}

template<class T> void FunctionNode<T>::print( ostream& output ) const {
	output << name_ << " ( ";
	for ( int i = 0; i < arity_; i++ ) {
		GPNode<T>& node = *inputs_[ i ];
		if ( i != 0 ) output << " ";
		output << node;
	}
	output << " ) " << flush;
}


template<class T> void FunctionNode<T>::read( istream& input ) {

	// TODO need to re-organise this so that
	// we can pass in a list of functions and terminals
	// to look at when building the tree up again
	
	char c;
	input >> c;
	assert( c == '(' );
	
	for ( int i = 0; i < arity_; i++ ) {
		char name[ 32 ];
		input >> name;
		FunctionNode<T>* node = makeFunction( name );
        if ( node != NULL ) {
			inputs_[ i ] = node;
		}
		else {
			istrstream strin( name );
			T c;
			strin >> c;
			inputs_[ i ] = new ConstantNode<T>( c );
		}
		input >> (*inputs_[ i ]);
	}
	
	input >> c;
	assert( c == ')' );

}

template<class T> void FunctionNode<T>::executeInputs() {
	for ( int i = 0; i < arity_; i++ ) inputs_[ i ]->execute();
}

#endif // __FunctionNode_H__
