#ifndef __GPProgram_H__
#define __GPProgram_H__

#include <iostream>
#include <vector>
#include <limits>
using namespace std;

#include "GPTree.h"
#include "Evaluation.h"
#include "ADFunction.h"
#include "rand-util.h"
#include "auto_list.h"

template<class T> class GPProgram {

public:
	GPProgram( int depth, auto_list< ADFunction<T> >& definedFunctions, bool initialise = true );
	virtual ~GPProgram();
	
public:
	int size() const;
	
	void evaluate( T& output, int function = 0 );
	
	int num_trees() { return functionTrees_.size(); };
	      GPTree<T>& tree( int i )       { return functionTrees_[ i ]; };
	const GPTree<T>& tree( int i ) const { return functionTrees_[ i ]; };
	
	void print( ostream& output ) const;
	void read( istream& input );

public:
	void operator=( const GPProgram<T>& rhs );
	bool operator==( const GPProgram<T>& rhs ) const;
	
public:
	static vector<int> DEFINED_FUNCTIONS;
	static vector<string> FUNCTION_SET;
	static vector<string> CONSTANTS;

protected:

protected:
	auto_list< GPTree<T> > functionTrees_;
	auto_list< ADFunction<T> >& definedFunctions_;
};

template<class T> vector<int>    GPProgram<T>::DEFINED_FUNCTIONS;
template<class T> vector<string> GPProgram<T>::FUNCTION_SET;
template<class T> vector<string> GPProgram<T>::CONSTANTS;


template<class T> GPProgram<T>::GPProgram( int depth, auto_list< ADFunction<T> >& definedFunctions, bool initialise )
 : definedFunctions_( definedFunctions )
{
	
	for ( unsigned int i = 0; i < definedFunctions_.size(); i++ ) {
		ADFunction<T>& fn = definedFunctions_[ i ];
		
		GPTree<T>* tree = new GPTree<T>( depth, fn.inputs(), fn.terminals(), fn.functions() );
		functionTrees_.add( tree );
		if ( initialise )
			tree->initialise( 1 + randrange( depth ), randdouble() < 0.5 );
	}
		
	assert( DEFINED_FUNCTIONS.size() == definedFunctions_.size() );
}

template<class T> GPProgram<T>::~GPProgram() {

}

template<class T> void GPProgram<T>::evaluate( T& output, int function ) {
	for ( unsigned int i = 0; i < definedFunctions_.size(); i++ ) {
		definedFunctions_[ i ].set( functionTrees_[ i ] );
	}
	functionTrees_[ function ].evaluate( output );
};

template<class T> int GPProgram<T>::size() const {
	int size = 0;
	for ( unsigned int i = 0; i < functionTrees_.size(); i++ ) {
		size += functionTrees_[ i ].size();
	}
	return size;
}


template<class T> void GPProgram<T>::operator=( const GPProgram<T>& rhs ) {
	if ( this == &rhs )
		return;
	for ( unsigned int i = 0; i < functionTrees_.size(); i++ ) {
		functionTrees_[ i ] = rhs.functionTrees_[ i ];
	}
}

template<class T> bool GPProgram<T>::operator==( const GPProgram<T>& rhs ) const {
	if ( this == &rhs )
		return true;
	
	for ( unsigned int i = 0; i < functionTrees_.size(); i++ ) {
		if ( !(functionTrees_[ i ] == rhs.functionTrees_[ i ]) )
			return false;
	}
	return true;
}

template<class T> void GPProgram<T>::print( ostream& output ) const {
	output << "(" << endl;
	
	for ( unsigned int i = 0; i < functionTrees_.size(); i++ ) {
		ADFunction<T>& fn = definedFunctions_[ i ];
		const GPTree<T>& tree = functionTrees_[ i ];
		output << "def " << fn.name() << " ( ";
		for ( int j = 0; j < fn.arity(); j++ ) {
			output << "input" << j << " ";
		}
		output << ")" << endl;
		output << tree;
	}
	output << ")" << endl;
}

template<class T> void GPProgram<T>::read( istream& input ) {
	char c;
	input >> c;
	assert( c == '(' );
	
	for ( unsigned int i = 0; i < functionTrees_.size(); i++ ) {
		cout << "ADF" << i << endl;
		ADFunction<T>& fn = definedFunctions_[ i ];
		GPTree<T>& tree = functionTrees_[ i ];
		string def;
		input >> def;
		assert( def == "def" );
		string fname;
		input >> fname;
		assert( fname == fn.name() );
		input >> c;
		assert( c == '(' );
		for ( int j = 0; j < fn.arity(); j++ ) {
			string inputName;
			input >> inputName;
		}
		input >> c;
		assert( c == ')' );
		
		input >> tree;
	}
	input >> c;
	assert( c == ')' );
	
}

template<class T> ostream& operator<<(ostream& output, const GPProgram<T>& program ) {
	program.print( output );
    	return output;
}

template<class T> istream& operator>>(istream& input, GPProgram<T>& program ) {
	program.read( input );
	return input;
}

#endif // __GPProgram_H__
