#ifndef __Image_H__
#define __Image_H__

#include "rand-util.h"

#include "opencv/cv.h"

#include <iostream>
using namespace std;



class Image {

public:
	static unsigned int DEPTH;

public:
	Image();
	Image( int w, int h, int depth = DEPTH );
	Image( const Image& img );
	Image( const char* filename );
	~Image();

public:
	void load( const char* filename );
		
public:
	void flood( double value, int x, int y, double lo=0.0, double up=0.0 );
	void fill( double value, int x, int y, int width, int height );
	void circle( double value, int x, int y, int radius, bool fill = true );
	void line( double value, int x1, int y1, int x2, int y2 );
	void blit( const Image& img, int x, int y );
	
	void add_noise( double range );
	//void add_array_noise( int* noise );
	
public:
	void operator=( const Image& rhs );
	void operator=( double rhs );
	void operator=( unsigned char* rhs );
	
	void convert( const Image& image );
	void scale( double scale, const Image& image );
	void resize( const Image& image );
	
	// image with image
	void add( const Image& a, const Image& b );
	void sub( const Image& a, const Image& b );
	void mul( const Image& a, const Image& b );
	void div( const Image& a, const Image& b );
	void gt( const Image& a, const Image& b );
	void lt( const Image& a, const Image& b );
	void max( const Image& a, const Image& b );
	void min( const Image& a, const Image& b );
	void absDiff( const Image& a, const Image& b );
	
	// image with double (scalar)
	void add( const Image& a, double b );
	void sub( const Image& a, double b );
	void mul( const Image& a, double b );
	void div( const Image& a, double b );
	void gt( const Image& a, double b );
	void lt( const Image& a, double b );
	void max( const Image& a, double b );
	void min( const Image& a, double b );
	void absDiff( const Image& a, double b );
	
	// double with image
	void add( double a, const Image& b );
	void sub( double a, const Image& b );
	void mul( double a, const Image& b );
	void div( double a, const Image& b );
	void gt( double a, const Image& b );
	void lt( double a, const Image& b );
	void max( double a, const Image& b );
	void min( double a, const Image& b );
	void absDiff( double a, const Image& b );
	
	void gaussian( const Image& image, int w, int h );
	void erode( const Image& image );
	void dilate( const Image& image );
	
	void canny( const Image& image, double threshold1, double threshold2 );
	void good_features( const Image& image, double qualityLevel, double minDistance );
	
	double avg() const;
	double sum() const;
	
	void maxLoc( int& x, int& y ) const;
	
	void centreOfGravity( int& x, int& y ) const;
public:

	int width()  const { return w_; };
	int height() const { return h_; };

	      IplImage* image()       { return image_; };
	const IplImage* image() const { return image_; };

protected:
	IplImage* image_;
	int w_;
	int h_;

};


#endif // __Image_H__
