#include "ImageArray.h"

int ImageArray::WIDTH  = 160;
int ImageArray::HEIGHT = 1;


ImageArray::ImageArray()
 : image_( NULL )
{
	image_ = popImage();
}

ImageArray::ImageArray( const ImageArray& img )
 : image_( NULL )
{
	image_ = popImage();
	*this = img;
}

ImageArray::~ImageArray() {
	if ( image_ != NULL ) {
		pushImage( image_ );
		image_ = NULL;
	}
}

double ImageArray::toScalar() const {
	switch( type_ ) {
		case SCALAR: return scalar_;
		case VECTOR: return vector_.magnitude();
		case IMAGE:  return image_->avg();
	}
	return scalar_;
}

vector2d ImageArray::toVector() const {
	switch( type_ ) {
		case SCALAR: {
			vector2d v;
			v = scalar_;
			return v;
		}
		case VECTOR: return vector_;
		case IMAGE: {
			int x = 0, y = 0;
			image_->maxLoc( x, y );
			return vector2d( x, y );
		}
	}
	return vector_;
}
	
/*void ImageArray::toImage( Image& img ) const {
	switch( type_ ) {
		case SCALAR:
			img = scalar_;
			break;
		case VECTOR:
			img = 0.0;
			img.fill( 255.0, (int)vector_.x, (int)vector_.y, 1, 1 );
			break;
		case IMAGE:
			img = *image_;
			break;
	}
}*/
	
void ImageArray::operator=( const ImageArray& rhs ) {
	switch( rhs.type_ ) {
		case SCALAR: *this = rhs.scalar_; break;
		case VECTOR: *this = rhs.vector_; break;
		case IMAGE:  *this = *rhs.image_; break;
	}
}

void ImageArray::operator=( double rhs ) {
	type_ = SCALAR;
	scalar_ = rhs;
	//type_ = IMAGE;
	//*image_ = rhs;
}

void ImageArray::operator=( vector2d rhs ) {
	type_ = VECTOR;
	vector_ = rhs;
}

void ImageArray::operator=( const Image& rhs ) {
	type_ = IMAGE;
	*image_ = rhs;
}

void ImageArray::add( const ImageArray& a, const ImageArray& b ) {
	binaryOp( ADD, a, b );
}

void ImageArray::sub( const ImageArray& a, const ImageArray& b ) {
	binaryOp( SUB, a, b );
}

void ImageArray::mul( const ImageArray& a, const ImageArray& b ) {
	binaryOp( MUL, a, b );
}

void ImageArray::div( const ImageArray& a, const ImageArray& b ) {
	binaryOp( DIV, a, b );
}

void ImageArray::gt( const ImageArray& a, const ImageArray& b ) {
	binaryOp( GT, a, b );
}

void ImageArray::lt( const ImageArray& a, const ImageArray& b ) {
	binaryOp( LT, a, b );
}

void ImageArray::max( const ImageArray& a, const ImageArray& b ) {
	binaryOp( MAX, a, b );
}

void ImageArray::min( const ImageArray& a, const ImageArray& b ) {
	binaryOp( MIN, a, b );
}

void ImageArray::diff( const ImageArray& a, const ImageArray& b ) {
	binaryOp( DIFF, a, b );
}

void ImageArray::abs( const ImageArray& a ) {
	switch( a.type_ ) {
		case SCALAR:
			*this = fabs( a.scalar_ );
		break;
		case VECTOR:
			*this = a.vector_.abs();
		break;
		case IMAGE:
			binaryOp( DIFF, *a.image_, 0.0 );
			type_ = IMAGE;
		break;
	}
}

void ImageArray::neg( const ImageArray& a ) {
	switch( a.type_ ) {
		case SCALAR:
			*this = -a.scalar_;
		break;
		case VECTOR:
			*this = 0.0-a.vector_;
		break;
		case IMAGE:
			binaryOp( SUB, 0.0, *a.image_ );
			type_ = IMAGE;
		break;
	}
}

void ImageArray::binaryOp( OP op, const ImageArray& a, const ImageArray& b ) {
	if ( a.type_ == IMAGE && b.type_ == IMAGE ) {
		binaryOp( op, *a.image_, *b.image_ );
		type_ = IMAGE;
	}
	else if ( a.type_ == VECTOR && b.type_ == VECTOR ) {
		binaryOp( op, a.vector_, b.vector_ );
		type_ = VECTOR;
	}
	else if ( a.type_ == SCALAR && b.type_ == SCALAR ) {
		binaryOp( op, a.scalar_, b.scalar_ );
		type_ = SCALAR;
	}
	else if ( a.type_ == IMAGE && b.type_ == SCALAR ) {
		binaryOp( op, *a.image_, b.toScalar() );
		type_ = IMAGE;
	}
	else if ( a.type_ == SCALAR && b.type_ == IMAGE ) {
		binaryOp( op, a.toScalar(), *b.image_ );
		type_ = IMAGE;
	}
	else if ( a.type_ == VECTOR || b.type_ == VECTOR ) {
		binaryOp( op, a.toVector(), b.toVector() );
		type_ = VECTOR;
	}
	else {
		*this = a;
	}
	/*else if ( a.type_ == IMAGE ) {
		binaryOp( op, *a.image_, b.toScalar() );
		type_ = IMAGE;
	}
	else if ( b.type_ == IMAGE ) {
		binaryOp( op, a.toScalar(), *b.image_ );
		type_ = IMAGE;
	}
	else if ( a.type_ == VECTOR ) {
		binaryOp( op, a.toVector(), b.toScalar() );
		type_ = VECTOR;
	}
	else if ( b.type_ == VECTOR ) {
		binaryOp( op, a.toScalar(), b.toVector() );
		type_ = VECTOR;
	}*/
}

void ImageArray::binaryOp( OP op, double a, double b ) {
	switch( op ) {
		case ADD:
			scalar_ = a + b;
			break;
		case SUB:
			scalar_ = a - b;
			break;
		case MUL:
			scalar_ = a * b;
			break;
		case DIV:
			scalar_ = b == 0.0? 1.0 : a / b;
			break;
		case GT:
			scalar_ = a > b;
			break;
		case LT:
			scalar_ = a < b;
			break;
		case MAX:
			scalar_ = a > b? a : b;
			break;
		case MIN:
			scalar_ = a < b? a : b;
			break;
		case DIFF:
			scalar_ = fabs( a - b );
			break;
	}
	*this = scalar_;
}

void ImageArray::binaryOp( OP op, vector2d a, vector2d b ) {
	switch( op ) {
		case ADD:
			vector_ = a + b;
			break;
		case SUB:
			vector_ = a - b;
			break;
		case MUL:
			vector_ = a * b;
			break;
		case DIV:
			vector_ = a / b;
			break;
		case GT:
			vector_.x = a.x > b.x;
			vector_.y = a.y > b.y;
			break;
		case LT:
			vector_.x = a.x < b.x;
			vector_.y = a.y < b.y;
			break;
		case MAX:
			vector_.x = a.x > b.x? a.x : b.x;
			vector_.y = a.y > b.y? a.y : b.y;
			break;
		case MIN:
			vector_.x = a.x < b.x? a.x : b.x;
			vector_.y = a.y < b.y? a.y : b.y;
			break;
		case DIFF:
			vector_ = (a - b).abs();
			break;
	}
	
	*this = vector_;
}

void ImageArray::binaryOp( OP op, const Image& a, const Image& b ) {
	switch( op ) {
		case ADD:
			image_->add( a, b );
			break;
		case SUB:
			image_->sub( a, b );
			break;
		case MUL:
			image_->mul( a, b );
			break;
		case DIV:
			image_->div( a, b );
			break;
		case GT:
			image_->gt( a, b );
			break;
		case LT:
			image_->lt( a, b );
			break;
		case MAX:
			image_->max( a, b );
			break;
		case MIN:
			image_->min( a, b );
			break;
		case DIFF:
			image_->absDiff( a, b );
			break;
	}
	*this = *image_;
}

void ImageArray::binaryOp( OP op, const Image& a, double b ) {
	switch( op ) {
		case ADD:
			image_->add( a, b );
			break;
		case SUB:
			image_->sub( a, b );
			break;
		case MUL:
			image_->mul( a, b );
			break;
		case DIV:
			image_->div( a, b );
			break;
		case GT:
			image_->gt( a, b );
			break;
		case LT:
			image_->lt( a, b );
			break;
		case MAX:
			image_->max( a, b );
			break;
		case MIN:
			image_->min( a, b );
			break;
		case DIFF:
			image_->absDiff( a, b );
			break;
	}
	*this = *image_;
}

void ImageArray::binaryOp( OP op, double a, const Image& b ) {
	switch( op ) {
		case ADD:
			image_->add( a, b );
			break;
		case SUB:
			image_->sub( a, b );
			break;
		case MUL:
			image_->mul( a, b );
			break;
		case DIV:
			image_->div( a, b );
			break;
		case GT:
			image_->gt( a, b );
			break;
		case LT:
			image_->lt( a, b );
			break;
		case MAX:
			image_->max( a, b );
			break;
		case MIN:
			image_->min( a, b );
			break;
		case DIFF:
			image_->absDiff( a, b );
			break;
	}
	*this = *image_;
}

/*void ImageArray::binaryOp( OP op, vector2d a, double b ) {
	switch( op ) {
		case ADD:
			vector_ = a + b;
			break;
		case SUB:
			vector_ = a - b;
			break;
		case MUL:
			vector_ = a * b;
			break;
		case DIV:
			vector_ = a / b;
			break;
		case GT:
			vector_.x = a.x > b;
			vector_.y = a.y > b;
			break;
		case LT:
			vector_.x = a.x < b;
			vector_.y = a.y < b;
			break;
		case MAX:
			vector_.x = a.x > b? a.x : b;
			vector_.y = a.y > b? a.y : b;
			break;
		case MIN:
			vector_.x = a.x < b? a.x : b;
			vector_.y = a.y < b? a.y : b;
			break;
		case DIFF:
			vector_ = (a - b).abs();
			break;
	}
	*this = vector_;
}

void ImageArray::binaryOp( OP op, double a, vector2d b ) {
	switch( op ) {
		case ADD:
			vector_ = a + b;
			break;
		case SUB:
			vector_ = a - b;
			break;
		case MUL:
			vector_ = a * b;
			break;
		case DIV:
			vector_ = a / b;
			break;
		case GT:
			vector_.x = a > b.x;
			vector_.y = a > b.y;
			break;
		case LT:
			vector_.x = a < b.x;
			vector_.y = a < b.y;
			break;
		case MAX:
			vector_.x = a > b.x? a : b.x;
			vector_.y = a > b.y? a : b.y;
			break;
		case MIN:
			vector_.x = a < b.x? a : b.x;
			vector_.y = a < b.y? a : b.y;
			break;
		case DIFF:
			vector_ = (a - b).abs();
			break;
	}
	*this = vector_;
}*/

memory_pool ImageArray::memory_pool_;

void* ImageArray::operator new(size_t size) {
	if ( memory_pool_.empty() )
		return ::operator new(size);
	return memory_pool_.pop();
}

void ImageArray::operator delete(void* ptr) {
	memory_pool_.push( ptr );
}

auto_list< Image > ImageArray::image_list_;

Image* ImageArray::popImage() {
	if ( image_list_.empty() )
		return new Image( WIDTH, HEIGHT ); 
	return image_list_.pop();
}

void ImageArray::pushImage( Image* image ) {
	image_list_.add( image );
}
