#ifndef __ImageArray_H__
#define __ImageArray_H__

#include "Image.h"
#include "auto_list.h"
#include "memory_pool.h"

#include "vector2d.h"

class ImageArray {

public:
	enum TYPE { SCALAR, VECTOR, IMAGE };
	enum OP { ADD, SUB, MUL, DIV, GT, LT, MAX, MIN, DIFF };
public:
	ImageArray();
	ImageArray( const ImageArray& img );
	~ImageArray();
	
public:
	TYPE type() const { return type_; };
	double   toScalar() const;
	vector2d toVector() const;
	//void toImage( Image& img ) const;
	Image& image() { return *image_; };
	const Image& image() const { return *image_; };

public:
	void operator=( const ImageArray& rhs );
	void operator=( double rhs );
	void operator=( vector2d rhs );
	void operator=( const Image& rhs );

public:
	void add( const ImageArray& a, const ImageArray& b );
	void sub( const ImageArray& a, const ImageArray& b );
	void mul( const ImageArray& a, const ImageArray& b );
	void div( const ImageArray& a, const ImageArray& b );
	void gt( const ImageArray& a, const ImageArray& b );
	void lt( const ImageArray& a, const ImageArray& b );
	void max( const ImageArray& a, const ImageArray& b );
	void min( const ImageArray& a, const ImageArray& b );
	void diff( const ImageArray& a, const ImageArray& b );

	void abs( const ImageArray& a );
	void neg( const ImageArray& a );

protected:
	void binaryOp( OP op, const ImageArray& a, const ImageArray& b );
	void binaryOp( OP op, double a, double b );
	void binaryOp( OP op, vector2d a, vector2d b );
	void binaryOp( OP op, const Image& a, const Image& b );
	void binaryOp( OP op, const Image& a, double b );
	void binaryOp( OP op, double a, const Image& b );
	//void binaryOp( OP op, vector2d a, double b );
	//void binaryOp( OP op, double a, vector2d b );
public:
	static int WIDTH;
	static int HEIGHT;
	
protected:
	TYPE type_;
	double scalar_;
	vector2d vector_;
	Image* image_;
	
protected:
	static auto_list< Image > image_list_;

public:
	static void* operator new(size_t size);
	static void operator delete(void* ptr);

public:
	static Image* popImage();
	static void pushImage( Image* image );
	
protected:
	static memory_pool memory_pool_;
				
};


#endif // __ImageArray_H__
