#include "ImageArray.h"

///////////////////////////////////////////////////////////
// Binary Operators
///////////////////////////////////////////////////////////
void add_op( const ImageArray& a, const ImageArray& b, ImageArray& output ) {
	output.add( a, b );
}

void sub_op( const ImageArray& a, const ImageArray& b, ImageArray& output ) {
	output.sub( a, b );
}

void mul_op( const ImageArray& a, const ImageArray& b, ImageArray& output ) {
	output.mul( a, b );
}

void div_op( const ImageArray& a, const ImageArray& b, ImageArray& output ) {
	output.div( a, b );
}

void gt_op( const ImageArray& a, const ImageArray& b, ImageArray& output ) {
	output.gt( a, b );
}

void lt_op( const ImageArray& a, const ImageArray& b, ImageArray& output ) {
	output.lt( a, b );
}

void max_op( const ImageArray& a, const ImageArray& b, ImageArray& output ) {
	output.max( a, b );
}

void min_op( const ImageArray& a, const ImageArray& b, ImageArray& output ) {
	output.min( a, b );
}

void diff_op( const ImageArray& a, const ImageArray& b, ImageArray& output ) {
	output.diff( a, b );
}

///////////////////////////////////////////////////////////
// Unary Operators
///////////////////////////////////////////////////////////

void abs_op( const ImageArray& input, ImageArray& output ) {
	output.abs( input );
}

void neg_op( const ImageArray& input, ImageArray& output ) {
	output.neg( input );
}

void avg_op( const ImageArray& input, ImageArray& output ) {
	if ( input.type() != ImageArray::IMAGE )
		output = input;
	else {
		output = input.toScalar();
	}
}	

// centroid
void cen_op( const ImageArray& input, ImageArray& output ) {
	if ( input.type() != ImageArray::IMAGE )
		output = input;
	else {
		const Image& img = input.image();
		int x = 0, y = 0;
		img.centreOfGravity( x, y );
		output = vector2d( x, y );
	}
}

void peak_op( const ImageArray& input, ImageArray& output ) {
	if ( input.type() != ImageArray::IMAGE )
		output = input;
	else {
		const Image& img = input.image();
		int x = 0, y = 0;
		img.maxLoc( x, y );
		output = vector2d( x, y );
	}
}

void gauss_op( const ImageArray& input, ImageArray& output ) {
	if ( input.type() != ImageArray::IMAGE )
		output = input;
	else {
		Image& img = output.image();
		img.gaussian( input.image(), 9, 9 );
		output = img;
	}
}

void erode_op( const ImageArray& input, ImageArray& output ) {
	if ( input.type() != ImageArray::IMAGE )
		output = input;
	else {
		Image& img = output.image();
		img.erode( input.image() );
		output = img;
	}
}

void dilate_op( const ImageArray& input, ImageArray& output ) {
	if ( input.type() != ImageArray::IMAGE )
		output = input;
	else {
		Image& img = output.image();
		img.dilate( input.image() );
		output = img;
	}
}

// ternary
void canny_op( const ImageArray& input, const ImageArray& threshold1, const ImageArray& threshold2, ImageArray& output ) {
	if ( input.type() != ImageArray::IMAGE )
		output = input;
	else {
		double t1 = fabs( threshold1.toScalar() );
		double t2 = fabs( threshold2.toScalar() );
		Image& img = output.image();
		img.canny( input.image(), t1, t2 );
		output = img;
	}
}

void features_op( const ImageArray& input, const ImageArray& quality, const ImageArray& distance, ImageArray& output ) {
	if ( input.type() != ImageArray::IMAGE )
		output = input;
	else {
		double q = fabs( quality.toScalar() );
		double d = fabs( distance.toScalar() );
		if ( q == 0.0 ) q = 0.0000001;
		if ( d == 0.0 ) d = 0.0000001;
		Image& img = output.image();
		img.good_features( input.image(), q, d );
		output = img;
	}
	
}
