#include "ImageMinimalSim.h"

#include "rand-util.h"

#ifdef SHOW_SIMULATION
#include "opencv/cv.h"
#include "opencv/highgui.h"
#endif

int ImageMinimalSim::NOISE = 30;

ImageMinimalSim::ImageMinimalSim()
 : background_( ImageArray::WIDTH, ImageArray::HEIGHT ),
   object_( max( ImageArray::WIDTH/10, 1 ), max( ImageArray::WIDTH/10, 1 ) ),
   start_x_( 0 ),
   start_y_( 0 ),
   buffer_( ImageArray::WIDTH, ImageArray::HEIGHT )
{
	initialise();
#ifdef SHOW_SIMULATION
	cvNamedWindow( "Simulation", 1 );
#endif
}

ImageMinimalSim::~ImageMinimalSim() {
#ifdef SHOW_SIMULATION
	cvDestroyWindow( "Simulation" );
#endif
}

void ImageMinimalSim::initialise() {
	int noise = NOISE;
	int bg = 0;
	int fg = 255;
	do {
		bg = randrange( 255-noise );
		fg = randrange( 255-noise );
	} while( abs( bg-fg ) < 10 );
	
	buffer_ = bg;
	
	for ( int i = 0; i < 30; i++ ) {
		int x = randrange( buffer_.width() );
		int y = randrange( buffer_.height() );
		int w = randrange( buffer_.width() );
		int h = randrange( buffer_.height() );
		
		buffer_.fill( bg + randrange( noise ), x, y, w, h );
	}
	
	buffer_.gaussian( buffer_, 9, 9 );
	
	start_x_ = randrange( ImageArray::WIDTH+1 );
	start_y_ = randrange( ImageArray::HEIGHT+1 );
	
	int size = buffer_.width()/20 + randrange( buffer_.width()/20 );
	
	for ( int i = 0; i < 30; i++ ) {
		int x = start_x_ + randrange( size ) - size/2;
		int y = start_y_ + randrange( size ) - size/2;
		buffer_.circle( fg + randrange( noise ), x, y, randrange( size ) );
	}
	
	// then add completely random dots
	for ( int i = 0; i < noise; i++ ) {
		int x = randrange( buffer_.width() );
		int y = randrange( buffer_.height() );
		buffer_.fill( randrange( 255 ), x, y, 1, 1 );
	}
}

void ImageMinimalSim::render( Image& buffer, int x, int y ) {
	/*int objectx = x;
	int objecty = y;
	
	// TODO wrap background
	int bg = -1;
	buffer.blit( background_, 0, 0 );
	buffer.blit( object_, objectx-(object_.width()/2), objecty-(object_.height()/2), &bg );*/
}

double ImageMinimalSim::evaluate( Candidate<ImageArray>& candidate ) {
	//return evaluateMovement( input0, input1, candidate );
	return evaluateStatic( candidate );
}

double ImageMinimalSim::evaluateStatic( Candidate<ImageArray>& candidate ) {
	int objectx = start_x_;
	int objecty = start_y_;
	
	//buffer_ = 0;
	//buffer_.circle( 255, objectx, objecty, buffer_.width()/10 );
	
	vector<ImageArray*> inputs = candidate.tree( 0 ).inputs();
	
	ImageArray& input0 = inputs[ 0 ];
		
	input0 = buffer_;
	
	input1 = input0;
	
	ImageArray result;
	candidate.program().evaluate( result );
	Image& result_img = result.image();
	result.toImage( result_img );
		
	int x = 0, y = 0;
	vector2d pos = result.toVector();
	x = (int)pos.x;
	y = (int)pos.y;
	//cout << x << " " << y << endl;
	
#ifdef SHOW_SIMULATION
	if ( randdouble() < 0.005) {
		Image& image = input0.image();
		image.line( 255.0, 0, y, image.width(), y );
		image.line( 255.0, x, 0, x, image.height() );
		cvShowImage("Simulation",image.image() );
		cvWaitKey(5);
	}
#endif
	
	int dx = objectx-x;
	int dy = objecty-y;
	
	int distSq = dx*dx + dy*dy;
	//int minDesDist = 1 + buffer_.width()/20;
	//int minDesDistSq = minDesDist*minDesDist;
	double maxDistSq = buffer_.width()*buffer_.width() + buffer_.height()*buffer_.height();
	//if ( distSq > (object_.width()*object_.width() + object_.height()*object_.height()) )
	//	return 10*distSq;
	//if ( distSq <= minDesDistSq )
	//	return 0.0;
	return distSq/maxDistSq;

}
