#ifndef __RootNode_H__
#define __RootNode_H__

#include "FunctionNode.h"
#include "BinaryOperator.h"

template<class T> class RootNode : public FunctionNode<T> {

public:
	RootNode();
	virtual ~RootNode();
	
public:
	virtual void execute();
	
protected:
	FunctionNode<T>* makeFunction( const char* name ) const;
	

};

///////////////////////
// implementation
///////////////////////

template<class T> RootNode<T>::RootNode()
 : FunctionNode<T>::FunctionNode( *this, "", 1 )
{

}

template<class T> RootNode<T>::~RootNode() {
}
	
template<class T> void RootNode<T>::execute() {
	GPNode<T>& node = *(inputs_[ 0 ]);
	node.execute();
	output_ = node.output();
}

template<class T> void add( const T& a, const T& b, T& output ) {
	output = a;
	output += b;
}

template<class T> void sub( const T& a, const T& b, T& output ) {
	output = a;
	output -= b;
}

template<class T> void mul( const T& a, const T& b, T& output ) {
	output = a;
	output *= b;
}

template<class T> void div( const T& a, const T& b, T& output ) {
	output = a;
	output /= b;  // TODO should be "safe" division
}

template<class T> FunctionNode<T>* RootNode<T>::makeFunction( const char* name ) const {
	if ( strcmp( name, "add" ) == 0 )
		return new BinaryOperator<T>( *this, "add", add );
	if ( strcmp( name, "sub" ) == 0 )
		return new BinaryOperator<T>( *this, "sub", sub );
	if ( strcmp( name, "mul" ) == 0 )
		return new BinaryOperator<T>( *this, "mul", mul );
	if ( strcmp( name, "div" ) == 0 )
		return new BinaryOperator<T>( *this, "div", div );
	return NULL;
}

#endif // __RootNode_H__
