#include "functions.h"

#include "ImageArrayOperators.h"
#include "UnaryOperator.h"
#include "BinaryOperator.h"
#include "TernaryOperator.h"

#include "GPProgram.h"

void create_functions( auto_list< Function<ImageArray> >& functions ) {
	for ( unsigned int i = 0; i < GPProgram<ImageArray>::FUNCTION_SET.size(); i++ ) {
		string fn = GPProgram<ImageArray>::FUNCTION_SET[ i ];
		
		if ( fn == "+" )
			functions.add( new BinaryOperator<ImageArray>( "+", add_op ) );
		else if ( fn == "-" )
			functions.add( new BinaryOperator<ImageArray>( "-", sub_op ) );
		else if ( fn == "*" )
			functions.add( new BinaryOperator<ImageArray>( "*", mul_op ) );
		else if ( fn == "/" )
			functions.add( new BinaryOperator<ImageArray>( "/", div_op ) );
		else if ( fn == ">" )
			functions.add( new BinaryOperator<ImageArray>( ">", gt_op ) );
		else if ( fn == "<" )
			functions.add( new BinaryOperator<ImageArray>( "<", lt_op ) );
		else if ( fn == "max" )
			functions.add( new BinaryOperator<ImageArray>( "max", max_op ) );
		else if ( fn == "min" )
			functions.add( new BinaryOperator<ImageArray>( "min", min_op ) );
		else if ( fn == "diff" )
			functions.add( new BinaryOperator<ImageArray>( "diff", diff_op ) );
		else if ( fn == "abs" )
			functions.add( new UnaryOperator<ImageArray> ( "abs", abs_op   ) );
		else if ( fn == "neg" )
			functions.add( new UnaryOperator<ImageArray> ( "neg", neg_op   ) );
		else if ( fn == "avg" )
			functions.add( new UnaryOperator<ImageArray> ( "avg", avg_op   ) );
		else if ( fn == "cen" )
			functions.add( new UnaryOperator<ImageArray> ( "cen",   cen_op   ) );
		else if ( fn == "peak" )
			functions.add( new UnaryOperator<ImageArray> ( "peak",   peak_op   ) );
		else if ( fn == "gauss" )
			functions.add( new UnaryOperator<ImageArray> ( "gauss", gauss_op ) );
		else if ( fn == "erode" )
			functions.add( new UnaryOperator<ImageArray> ( "erode", erode_op ) );
		else if ( fn == "dilate" )
			functions.add( new UnaryOperator<ImageArray> ( "dilate", dilate_op ) );
		else if ( fn == "canny" )
			functions.add( new TernaryOperator<ImageArray> ( "canny", canny_op ) );
		else if ( fn == "features" )
			functions.add( new TernaryOperator<ImageArray> ( "features", features_op ) );
		else
			cerr << fn << " unknown function" << endl;
	}
}
