#include "init.h"

#include "Candidate.h"
#include "SteadyStateGA.h"
#include "MinimalSimEvaluation.h"
//#include "ImagesEvaluation.h"
#include "Properties.h"

#include <string>
#include <sstream>
using namespace std;


extern int TEST_SEED;
extern int TEST_SIZE;

vector<string> split( string& str ) {
	vector<string> tokens;
	while ( str.size() > 0 ) {
		string token = "";
		while ( str.size() > 0 ) {
			char c = str[ 0 ];
			if ( c != ',' ) {
				token += c;
			}
			str.assign( str, 1, str.size()-1 );
			if ( c == ',' )
				break;
		}
		tokens.push_back( token );
	}
	return tokens;
}

void read_params( istream& input ) {
	Properties params( input );

	Candidate<ImageArray>::MAX_DEPTH         = params.get( "Candidate::MAX_DEPTH",       Candidate<ImageArray>::MAX_DEPTH );
	Candidate<ImageArray>::MUTATE_SUBTREE    = params.get( "Candidate::MUTATE_SUBTREE",  Candidate<ImageArray>::MUTATE_SUBTREE );
	Candidate<ImageArray>::MUTATE_TERMINAL   = params.get( "Candidate::MUTATE_TERMINAL", Candidate<ImageArray>::MUTATE_TERMINAL );
	Candidate<ImageArray>::MUTATE_ROOT       = params.get( "Candidate::MUTATE_ROOT",     Candidate<ImageArray>::MUTATE_ROOT );
	Candidate<ImageArray>::MUTATE_FUNCTION   = params.get( "Candidate::MUTATE_FUNCTION", Candidate<ImageArray>::MUTATE_FUNCTION );
	
	int numADFs = params.get( "GPProgram::DEFINED_FUNCTIONS", 0 );
	for ( int i = 0; i < numADFs; i++ ) {
		ostringstream ostr;
		ostr << "GPProgram::FN" << i << flush;
		string key = ostr.str();
		int numArgs = params.get( key.c_str(), 0 );
		GPProgram<ImageArray>::DEFINED_FUNCTIONS.push_back( numArgs );
	}
	string constants = params.get( "GPProgram::CONSTANTS", "0,1" );
	GPProgram<ImageArray>::CONSTANTS = split( constants );
	string functions = params.get( "GPProgram::FUNCTION_SET", "+,-,*,/,<,>,max,min,diff,not,avg,cen,gauss,erode,dilate,canny" );
	GPProgram<ImageArray>::FUNCTION_SET = split( functions );
	
	
	
	SteadyStateGA<ImageArray>::POP_SIZE             = params.get( "SteadyStateGA::POP_SIZE",            SteadyStateGA<ImageArray>::POP_SIZE );
	SteadyStateGA<ImageArray>::ITERATIONS           = params.get( "SteadyStateGA::ITERATIONS",          SteadyStateGA<ImageArray>::ITERATIONS );
	SteadyStateGA<ImageArray>::TOURNAMENT_SIZE      = params.get( "SteadyStateGA::TOURNAMENT_SIZE",     SteadyStateGA<ImageArray>::TOURNAMENT_SIZE );
	SteadyStateGA<ImageArray>::RECOMBINATION        = params.get( "SteadyStateGA::RECOMBINATION",       SteadyStateGA<ImageArray>::RECOMBINATION );
	SteadyStateGA<ImageArray>::MUTATION             = params.get( "SteadyStateGA::MUTATION",            SteadyStateGA<ImageArray>::MUTATION );
	SteadyStateGA<ImageArray>::REPLACE              = params.get( "SteadyStateGA::REPLACE",             SteadyStateGA<ImageArray>::REPLACE );
	SteadyStateGA<ImageArray>::EXPECTED_ERROR       = params.get( "SteadyStateGA::EXPECTED_ERROR",      SteadyStateGA<ImageArray>::EXPECTED_ERROR );
	SteadyStateGA<ImageArray>::EXPECTED_COMPLEXITY  = params.get( "SteadyStateGA::EXPECTED_COMPLEXITY", SteadyStateGA<ImageArray>::EXPECTED_COMPLEXITY );
	
	ImageArray::WIDTH                        = params.get( "ImageArray::WIDTH",  ImageArray::WIDTH );
	ImageArray::HEIGHT                       = params.get( "ImageArray::HEIGHT", ImageArray::HEIGHT  );
	
	Evaluation<ImageArray>::TEST_CASES       = params.get( "Evaluation::TEST_CASES",   Evaluation<ImageArray>::TEST_CASES );
	Evaluation<ImageArray>::EPOCH_LENGTH     = params.get( "Evaluation::EPOCH_LENGTH", Evaluation<ImageArray>::EPOCH_LENGTH );
	Evaluation<ImageArray>::EVAL_FUNCTION    = params.get( "Evaluation::EVAL_FUNCTION", Evaluation<ImageArray>::EVAL_FUNCTION );
	
	MinimalSimEvaluation::DATA_FILE          = params.get( "MinimalSimEvaluation::DATA_FILE", MinimalSimEvaluation::DATA_FILE );
	MinimalSimulation::NOISE                 = params.get( "MinimalSimulation::NOISE", MinimalSimulation::NOISE );
	MotionMinimalSim::PURE_MOTION            = params.get( "MotionMinimalSim::PURE_MOTION", MotionMinimalSim::PURE_MOTION );
	
	//ImagesEvaluation::IMAGES_DIR             = params.get( "ImagesEvaluation::IMAGES_DIR", ImagesEvaluation::IMAGES_DIR );
	
	TEST_SEED                                = params.get( "TEST_SEED", TEST_SEED );
	TEST_SIZE                                = params.get( "TEST_SIZE", TEST_SIZE );
	
		
}
