#include <iostream>
#include <fstream>
#include <vector>
#include <limits>
using namespace std;

#include "GPNode.h"
#include "Function.h"
#include "Constant.h"
#include "Variable.h"
#include "BinaryOperator.h"
#include "GPTree.h"
#include "SteadyStateGA.h"
#include "Evaluation.h"
#include "Candidate.h"
#include "auto_list.h"

//#include "TestInt.h"
//#include "TestImage.h"
//#include "TestImageOps.h"

#include "ImageArray.h"

#include "init.h"
#include "terminals.h"
#include "functions.h"
#include "adfs.h"
#include "MinimalSimEvaluation.h"
//#include "ImagesEvaluation.h"

int TEST_SEED = 0;
int TEST_SIZE = 100;

int main (int argc, const char * argv[]) {
	seed_rand_time();
	TEST_SEED = rand();
	//testint(); return 0;
	//test_image(); return 0;
	//test_image_ops(); return 0;
	bool params_read = false;
	bool quiet       = false;
	for ( int i = 1; i < argc; i++ ) {
		const char* arg = argv[ i ];
		if ( strcmp( arg, "-f" ) == 0 ) {
			if ( (i+1) < argc ) {
				ifstream infile( argv[ i+1 ] );
				read_params( infile );
				infile.close();
				params_read = true;
			}
			else {
				cout << "Usage: "  << argv[ 0 ] << " -f <params-file>" << endl;
				exit( -1 );
			}
		}
		else if ( strcmp( arg, "-q" ) == 0 ) {
			quiet = true;
		}
	}
	
	if ( !params_read ) {
		try {
			ifstream infile( "params.txt" );
			read_params( infile );
			infile.close();
		}
		catch( ... ){}
	}
		
	auto_list< ADFunction<ImageArray> > definedFunctions;
	
	create_adfs( GPProgram<ImageArray>::DEFINED_FUNCTIONS, definedFunctions );
	
	auto_ptr< Evaluation<ImageArray> > eval;
	
	/*if ( Evaluation<ImageArray>::EVAL_FUNCTION == "ImagesEvaluation" ) {
		auto_ptr< Evaluation<ImageArray> > imagesEval( new ImagesEvaluation( ImagesEvaluation::IMAGES_DIR.c_str() ) ); 
		eval = imagesEval;
	}
	else*/
	if ( Evaluation<ImageArray>::EVAL_FUNCTION == "MinimalSimEvaluation" ) {
		auto_ptr< Evaluation<ImageArray> > minimalEval( new MinimalSimEvaluation() ); 
		eval = minimalEval;
	}
	else {
		cerr << "Evaluation::EVAL_FUNCTION: " << Evaluation<ImageArray>::EVAL_FUNCTION << " unknown" << endl;
	}
	
	int num_iterations = SteadyStateGA<ImageArray>::ITERATIONS;
	int pop_size = SteadyStateGA<ImageArray>::POP_SIZE;
	int epoch_length = Evaluation<ImageArray>::EPOCH_LENGTH;
	
	SteadyStateGA<ImageArray> ga( *eval, pop_size, definedFunctions );
	//ga.recalculate_best();
	double currentScore = FLT_MAX;
	int currentSize    = -1;
	
	for ( int i = 0; i < num_iterations; i++ ) {
		ga.do_step();
		if ( !quiet && (currentScore != ga.best_fitness() || currentSize != ga.best_program_size()) ) {
			currentScore = ga.best_fitness();
			currentSize  = ga.best_program_size();
			cout << i << ",";
			cout << ga.best_fitness()   << ",";
			cout << ga.best_program_size() << endl;
		}
		if ( i % epoch_length == 0 && i != 0 ) {
			ga.force_evaluations();
			eval->initialise();
		} 
	}
	
	if ( !quiet ) {
		cout << num_iterations << ",";
		cout << ga.best_fitness() << ",";
		cout << ga.best_program_size() << endl;
		cout << "Av Fitness=" << ga.average_fitness()      << endl;
		cout << "Av Size="    << ga.average_program_size() << endl;
		cout << endl;
		cout << ga.best() << endl;
	}
	
	ga.force_evaluations();
	srand( TEST_SEED );
	eval->initialise( TEST_SIZE ); // use a large test set for final score
	ga.recalculate_best();
	
	if ( !quiet ) {
		cout << num_iterations << ",";
		cout << ga.best_fitness() << ",";
		cout << ga.best_program_size() << endl;
		cout << endl;
		cout << ga.best()<< endl;
	}
	else {
		cout << ga.best_fitness() << ", " << ga.best_program_size() << endl;
	}

	return 0;
}
