#include "vector2d.h"

#include <math.h>

vector2d::vector2d()
 : x( 0.0 ),
   y( 0.0 )
{

}

vector2d::vector2d( double nx, double ny )
 : x( nx ),
   y( ny )
{

}


double vector2d::dotProduct( const vector2d& v ) const {
	return x*v.x + y*v.y;
}

double vector2d::dotProduct() const {
	return dotProduct( *this );
}

double vector2d::magnitude() const {
	return sqrt( dotProduct() );
}
	
const vector2d& vector2d::operator=( double s ) {
	x = y = s;
	return *this;
}
	
vector2d vector2d::operator+( const vector2d& v ) const {
	return vector2d( x+v.x, y+v.y );
}

vector2d vector2d::operator-( const vector2d& v ) const {
	return vector2d( x-v.x, y-v.y );
}

vector2d vector2d::operator*( const vector2d& v ) const {
	return vector2d( x*v.x, y*v.y );
}

vector2d vector2d::operator/( const vector2d& v ) const {
	double nx = v.x == 0.0? 1.0 : x/v.x;
	double ny = v.y == 0.0? 1.0 : y/v.y;
	return vector2d( nx, ny );
}

vector2d vector2d::abs() const {
	return vector2d( fabs( x ), fabs( y ) );
}

vector2d vector2d::operator+( double s ) const {
	return operator+( vector2d( s, s ) );
}

vector2d vector2d::operator-( double s ) const {
	return operator-( vector2d( s, s ) );
}

vector2d vector2d::operator*( double s ) const  {
	return operator*( vector2d( s, s ) );
}

vector2d vector2d::operator/( double s ) const {
	return operator/( vector2d( s, s ) );
}

vector2d operator+( double s, const vector2d& v ) {
	return vector2d( s, s ).operator+( v );
}

vector2d operator-( double s, const vector2d& v )  {
	return vector2d( s, s ).operator-( v );
}

vector2d operator*( double s, const vector2d& v )  {
	return vector2d( s, s ).operator*( v );
}

vector2d operator/( double s, const vector2d& v )  {
	return vector2d( s, s ).operator/( v );
}

istream& operator>>( istream& input, vector2d& v ) {
	char b1;
	input >> b1;
	assert( b1 == '[' );
	string x, y;
	char c = -1;
	do {
		input >> c;
		if ( c == ',' || input.eof() )
			break;
		x += c;
	}
	while( true );
	
	do {
		input >> c;
		if ( c == ']' || input.eof() )
			break;
		y += c;
	}
	while( true );
	
	v.x = atof( x.c_str() );
	v.y = atof( y.c_str() );
	
	return input;
}

ostream& operator<<( ostream& output, const vector2d& v ) {
	output << "[ " << v.x << " , " << v.y << " ]";
	return output;
}
